import { isPlainObject, isString } from '@sindresorhus/is';
import { validateVersionNumber } from './version-number.js';
import { getGithubRepo } from './get-github-repo.js';
import { defaultValidLabels } from './valid-labels.js';
function stripTrailingEmptyLine(text) {
    if (text.endsWith('\n\n')) {
        return text.slice(0, -1);
    }
    return text;
}
function getValidLabels(packageInfo) {
    const prLogConfig = packageInfo['pr-log'];
    if (isPlainObject(prLogConfig) && Array.isArray(prLogConfig.validLabels)) {
        return new Map(prLogConfig.validLabels);
    }
    return defaultValidLabels;
}
export function createCliRunner(dependencies) {
    const { ensureCleanLocalGitState, getMergedPullRequests, createChangelog, packageInfo, prependFile, logger } = dependencies;
    async function generateChangelog(options, githubRepo, validLabels) {
        if (!options.sloppy) {
            await ensureCleanLocalGitState(githubRepo);
        }
        const mergedPullRequests = await getMergedPullRequests(githubRepo, validLabels);
        const commonChangelogOptions = { validLabels, mergedPullRequests, githubRepo };
        const changelogOptions = options.unreleased
            ? { ...commonChangelogOptions, unreleased: true, versionNumber: options.versionNumber }
            : { ...commonChangelogOptions, unreleased: false, versionNumber: options.versionNumber };
        const changelog = createChangelog(changelogOptions);
        return stripTrailingEmptyLine(changelog);
    }
    async function writeChangelog(changelog, options) {
        const trimmedChangelog = changelog.trim();
        if (options.stdout) {
            logger.log(trimmedChangelog);
        }
        else {
            await prependFile(options.changelogPath, `${trimmedChangelog}\n\n`);
        }
    }
    return {
        async run(options) {
            const { repository } = packageInfo;
            if (!isPlainObject(repository)) {
                throw new Error('Repository information missing in package.json');
            }
            if (!isString(repository.url)) {
                throw new TypeError('Repository url is not a string in package.json');
            }
            const githubRepo = getGithubRepo(repository.url);
            const validLabels = getValidLabels(packageInfo);
            validateVersionNumber(options).unwrapOrElse((error) => {
                throw error;
            });
            const changelog = await generateChangelog(options, githubRepo, validLabels);
            await writeChangelog(changelog, options);
        }
    };
}
//# sourceMappingURL=cli.js.map