import { format as formatDate } from 'date-fns';
import { isPlainObject, isArray, isString } from '@sindresorhus/is';
import enLocale from 'date-fns/locale/en-US/index.js';
function formatLinkToPullRequest(pullRequestId, repo) {
    return `[#${pullRequestId}](https://github.com/${repo}/pull/${pullRequestId})`;
}
function formatPullRequest(pullRequest, repo) {
    return `* ${pullRequest.title} (${formatLinkToPullRequest(pullRequest.id, repo)})\n`;
}
function formatListOfPullRequests(pullRequests, repo) {
    return pullRequests
        .map((pr) => {
        return formatPullRequest(pr, repo);
    })
        .join('');
}
function formatSection(displayLabel, pullRequests, repo) {
    return `### ${displayLabel}\n\n${formatListOfPullRequests(pullRequests, repo)}\n`;
}
function getConfigValueFromPackageInfo(packageInfo, fieldName, fallback) {
    const prLogConfig = packageInfo['pr-log'];
    if (isPlainObject(prLogConfig)) {
        const field = prLogConfig[fieldName];
        if (isString(field)) {
            return field;
        }
    }
    return fallback;
}
function groupByLabel(pullRequests) {
    return pullRequests.reduce((groupedObject, pullRequest) => {
        const { label } = pullRequest;
        const group = groupedObject[label];
        if (isArray(group)) {
            return {
                ...groupedObject,
                [label]: [...group, pullRequest]
            };
        }
        return {
            ...groupedObject,
            [label]: [pullRequest]
        };
    }, {});
}
const defaultDateFormat = 'MMMM d, yyyy';
export function createChangelogFactory(dependencies) {
    const { getCurrentDate, packageInfo } = dependencies;
    const dateFormat = getConfigValueFromPackageInfo(packageInfo, 'dateFormat', defaultDateFormat);
    function createChangelogTitle(options) {
        const { unreleased } = options;
        if (unreleased) {
            return '';
        }
        const date = formatDate(getCurrentDate(), dateFormat, { locale: enLocale });
        const title = `## ${options.versionNumber.value} (${date})`;
        return `${title}\n\n`;
    }
    return function createChangelog(options) {
        const { validLabels, mergedPullRequests, githubRepo } = options;
        const groupedPullRequests = groupByLabel(mergedPullRequests);
        let changelog = createChangelogTitle(options);
        for (const [label, displayLabel] of validLabels) {
            const pullRequests = groupedPullRequests[label];
            if (isArray(pullRequests)) {
                changelog += formatSection(displayLabel, pullRequests, githubRepo);
            }
        }
        return changelog;
    };
}
//# sourceMappingURL=create-changelog.js.map